@@ -1,19 +1,19 @@
 /*
  * Copyright (C) 2003,2008 Red Hat, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2.1 of the License, or (at your option) any later version.
+ * This is free software; you can redistribute it and/or modify it under
+ * the terms of the GNU Library General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
+ * General Public License for more details.
  *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this program; if not, write to the Free Software
+ * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 
@@ -24,6 +24,7 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 #include "debug.h"
+#include "vtebg.h"
 #include "vtedraw.h"
 #include "vte-private.h"
 
@@ -367,9 +368,6 @@ font_info_measure_font (struct font_info *info)
 	/* We don't do CEIL for width since we are averaging;
 	 * rounding is more accurate */
 	info->width  = PANGO_PIXELS (howmany (logical.width, strlen(VTE_DRAW_SINGLE_WIDE_CHARACTERS)));
-        /* Guard against pathological font since width=0 causes a FPE later on */
-	info->width = MAX (info->width, 1);
-
 	info->height = PANGO_PIXELS_CEIL (logical.height);
 	info->ascent = PANGO_PIXELS_CEIL (pango_layout_get_baseline (info->layout));
 
@@ -588,6 +586,7 @@ font_info_find_for_context (PangoContext *context)
 static struct font_info *
 font_info_create_for_context (PangoContext               *context,
 			      const PangoFontDescription *desc,
+			      VteTerminalAntiAlias        antialias,
 			      PangoLanguage              *language,
 			      guint                       fontconfig_timestamp)
 {
@@ -608,16 +607,37 @@ font_info_create_for_context (PangoContext               *context,
 
 	pango_context_set_language (context, language);
 
-        /* Make sure our contexts have a font_options set.  We use
-          * this invariant in our context hash and equal functions.
-          */
-        if (!pango_cairo_context_get_font_options (context)) {
-                cairo_font_options_t *font_options;
+	switch (antialias) {
+		cairo_font_options_t *font_options;
+		cairo_antialias_t cr_aa;
 
-                font_options = cairo_font_options_create ();
-                pango_cairo_context_set_font_options (context, font_options);
-                cairo_font_options_destroy (font_options);
-        }
+	case VTE_ANTI_ALIAS_FORCE_ENABLE:
+	case VTE_ANTI_ALIAS_FORCE_DISABLE:
+
+		if (antialias == VTE_ANTI_ALIAS_FORCE_ENABLE)
+			cr_aa = CAIRO_ANTIALIAS_DEFAULT; /* let surface decide between gray and subpixel */
+		else
+			cr_aa = CAIRO_ANTIALIAS_NONE;
+
+		font_options = cairo_font_options_copy (pango_cairo_context_get_font_options (context));
+		cairo_font_options_set_antialias (font_options, cr_aa);
+		pango_cairo_context_set_font_options (context, font_options);
+		cairo_font_options_destroy (font_options);
+
+		break;
+
+	default:
+	case VTE_ANTI_ALIAS_USE_DEFAULT:
+		/* Make sure our contexts have a font_options set.  We use
+		 * this invariant in our context hash and equal functions.
+		 */
+		if (!pango_cairo_context_get_font_options (context)) {
+			font_options = cairo_font_options_create ();
+			pango_cairo_context_set_font_options (context, font_options);
+			cairo_font_options_destroy (font_options);
+		}
+		break;
+	}
 
 	return font_info_find_for_context (context);
 }
@@ -625,23 +645,25 @@ font_info_create_for_context (PangoContext               *context,
 static struct font_info *
 font_info_create_for_screen (GdkScreen                  *screen,
 			     const PangoFontDescription *desc,
+			     VteTerminalAntiAlias        antialias,
 			     PangoLanguage              *language)
 {
 	GtkSettings *settings = gtk_settings_get_for_screen (screen);
 	int fontconfig_timestamp;
 	g_object_get (settings, "gtk-fontconfig-timestamp", &fontconfig_timestamp, NULL);
 	return font_info_create_for_context (gdk_pango_context_get_for_screen (screen),
-					     desc, language, fontconfig_timestamp);
+					     desc, antialias, language, fontconfig_timestamp);
 }
 
 static struct font_info *
 font_info_create_for_widget (GtkWidget                  *widget,
-			     const PangoFontDescription *desc)
+			     const PangoFontDescription *desc,
+			     VteTerminalAntiAlias        antialias)
 {
 	GdkScreen *screen = gtk_widget_get_screen (widget);
 	PangoLanguage *language = pango_context_get_language (gtk_widget_get_pango_context (widget));
 
-	return font_info_create_for_screen (screen, desc, language);
+	return font_info_create_for_screen (screen, desc, antialias, language);
 }
 
 static struct unistr_info *
@@ -722,47 +744,35 @@ font_info_get_unistr_info (struct font_info *info,
 	return uinfo;
 }
 
-guint _vte_draw_get_style(gboolean bold, gboolean italic) {
-	guint style = 0;
-	if (bold)
-		style |= VTE_DRAW_BOLD;
-	if (italic)
-		style |= VTE_DRAW_ITALIC;
-	return style;
-}
-
 struct _vte_draw {
 	GtkWidget *widget;
 
-	struct font_info *fonts[4];
+	gint started;
+
+	struct font_info *font;
+	struct font_info *font_bold;
 	cairo_pattern_t *bg_pattern;
 
 	cairo_t *cr;
 };
 
 struct _vte_draw *
-_vte_draw_new (void)
+_vte_draw_new (GtkWidget *widget)
 {
 	struct _vte_draw *draw;
 
 	/* Create the structure. */
 	draw = g_slice_new0 (struct _vte_draw);
+	draw->widget = g_object_ref (widget);
 
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_new\n");
 
 	return draw;
 }
 
-cairo_t *
-_vte_draw_get_context (struct _vte_draw *draw)
-{
-  return draw->cr;
-}
-
 void
 _vte_draw_free (struct _vte_draw *draw)
 {
-	gint style;
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_free\n");
 
 	if (draw->bg_pattern != NULL) {
@@ -770,44 +780,120 @@ _vte_draw_free (struct _vte_draw *draw)
 		draw->bg_pattern = NULL;
 	}
 
-	/* Free all fonts (make sure to destroy every font only once)*/
-	for (style = 3; style >= 0; style--) {
-		if (draw->fonts[style] != NULL &&
-			(style == 0 || draw->fonts[style] != draw->fonts[style-1])) {
-			font_info_destroy (draw->fonts[style]);
-			draw->fonts[style] = NULL;
-		}
+	if (draw->font != NULL) {
+		font_info_destroy (draw->font);
+		draw->font = NULL;
+	}
+
+	if (draw->widget != NULL) {
+		g_object_unref (draw->widget);
 	}
 
 	g_slice_free (struct _vte_draw, draw);
 }
 
 void
-_vte_draw_set_cairo (struct _vte_draw *draw,
-                     cairo_t *cr)
+_vte_draw_start (struct _vte_draw *draw)
 {
-        _vte_debug_print (VTE_DEBUG_DRAW, "%s cairo context\n", cr ? "Settings" : "Unsetting");
+	GdkWindow *window;
 
-        if (cr) {
-                g_assert (draw->cr == NULL);
-                draw->cr = cr;
-        } else {
-                g_assert (draw->cr != NULL);
-                draw->cr = NULL;
-        }
+	g_return_if_fail (gtk_widget_get_realized (draw->widget));
+
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_start\n");
+
+	if (draw->started == 0) {
+		window = gtk_widget_get_window(draw->widget);
+		g_object_ref (window);
+		draw->cr = gdk_cairo_create (window);
+	}
+
+	draw->started++;
+}
+
+void
+_vte_draw_end (struct _vte_draw *draw)
+{
+	g_return_if_fail (draw->started);
+
+	draw->started--;
+	if (draw->started == 0) {
+		cairo_destroy (draw->cr);
+		draw->cr = NULL;
+		g_object_unref (gtk_widget_get_window(draw->widget));
+ 	}
+
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_end\n");
 }
 
 void
 _vte_draw_set_background_solid(struct _vte_draw *draw,
-                               const GdkRGBA *color)
+			       double red,
+			       double green,
+			       double blue,
+			       double opacity)
 {
 	if (draw->bg_pattern)
 		cairo_pattern_destroy (draw->bg_pattern);
 
-	draw->bg_pattern = cairo_pattern_create_rgba (color->red,
-						      color->green,
-						      color->blue,
-						      color->alpha);
+	draw->bg_pattern = cairo_pattern_create_rgba (red,
+						      green,
+						      blue,
+						      opacity);
+}
+
+void
+_vte_draw_set_background_image (struct _vte_draw *draw,
+			        VteBgSourceType type,
+			        GdkPixbuf *pixbuf,
+			        const char *filename,
+			        const PangoColor *color,
+			        double saturation)
+{
+	cairo_surface_t *surface;
+
+	/* Need a valid draw->cr for cairo_get_target () */
+	_vte_draw_start (draw);
+
+	surface = vte_bg_get_surface (vte_bg_get_for_screen (gtk_widget_get_screen (draw->widget)),
+				     type, pixbuf, filename,
+				     color, saturation,
+				     cairo_get_target(draw->cr));
+
+	_vte_draw_end (draw);
+
+	if (!surface)
+		return;
+
+	if (draw->bg_pattern)
+		cairo_pattern_destroy (draw->bg_pattern);
+
+	draw->bg_pattern = cairo_pattern_create_for_surface (surface);
+	cairo_surface_destroy (surface);
+	cairo_pattern_set_extend (draw->bg_pattern, CAIRO_EXTEND_REPEAT);
+}
+
+void
+_vte_draw_set_background_scroll (struct _vte_draw *draw,
+				 gint x, gint y)
+{
+	cairo_matrix_t matrix;
+
+	_vte_debug_print (VTE_DEBUG_DRAW,
+			"draw_set_scroll (%d, %d)\n",
+			x, y);
+
+	g_return_if_fail (draw->bg_pattern != NULL);
+
+	cairo_matrix_init_translate (&matrix, x, y);
+	cairo_pattern_set_matrix (draw->bg_pattern, &matrix);
+}
+
+void
+_vte_draw_clip (struct _vte_draw *draw, GdkRegion *region)
+{
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clip\n");
+	gdk_cairo_region(draw->cr, region);
+	cairo_clip (draw->cr);
 }
 
 void
@@ -818,7 +904,6 @@ _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height
 	_vte_debug_print (VTE_DEBUG_DRAW, "draw_clear (%d, %d, %d, %d)\n",
 			  x,y,width, height);
 
-        g_assert(draw->cr);
 	cairo_rectangle (draw->cr, x, y, width, height);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_SOURCE);
 	cairo_set_source (draw->cr, draw->bg_pattern);
@@ -827,66 +912,32 @@ _vte_draw_clear (struct _vte_draw *draw, gint x, gint y, gint width, gint height
 
 void
 _vte_draw_set_text_font (struct _vte_draw *draw,
-                         GtkWidget *widget,
-                         const PangoFontDescription *fontdesc)
+			const PangoFontDescription *fontdesc,
+			VteTerminalAntiAlias antialias)
 {
-	PangoFontDescription *bolddesc   = NULL;
-	PangoFontDescription *italicdesc = NULL;
-	PangoFontDescription *bolditalicdesc = NULL;
-	gint style, normal, bold, ratio;
+	PangoFontDescription *bolddesc = NULL;
 
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font\n");
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_set_text_font (aa=%d)\n",
+			  antialias);
 
-	/* Free all fonts (make sure to destroy every font only once)*/
-	for (style = 3; style >= 0; style--) {
-		if (draw->fonts[style] != NULL &&
-			(style == 0 || draw->fonts[style] != draw->fonts[style-1])) {
-			font_info_destroy (draw->fonts[style]);
-			draw->fonts[style] = NULL;
-		}
-	}
+	if (draw->font_bold != draw->font)
+		font_info_destroy (draw->font_bold);
+	font_info_destroy (draw->font);
+	draw->font = font_info_create_for_widget (draw->widget, fontdesc, antialias);
 
 	/* calculate bold font desc */
 	bolddesc = pango_font_description_copy (fontdesc);
 	pango_font_description_set_weight (bolddesc, PANGO_WEIGHT_BOLD);
 
-	/* calculate italic font desc */
-	italicdesc = pango_font_description_copy (fontdesc);
-	pango_font_description_set_style (italicdesc, PANGO_STYLE_ITALIC);
-
-	/* calculate bold italic font desc */
-	bolditalicdesc = pango_font_description_copy (bolddesc);
-	pango_font_description_set_style (bolditalicdesc, PANGO_STYLE_ITALIC);
-
-	draw->fonts[VTE_DRAW_NORMAL]  = font_info_create_for_widget (widget, fontdesc);
-	draw->fonts[VTE_DRAW_BOLD]    = font_info_create_for_widget (widget, bolddesc);
-	draw->fonts[VTE_DRAW_ITALIC]  = font_info_create_for_widget (widget, italicdesc);
-	draw->fonts[VTE_DRAW_ITALIC | VTE_DRAW_BOLD] =
-                font_info_create_for_widget (widget, bolditalicdesc);
+	draw->font_bold = font_info_create_for_widget (draw->widget, bolddesc, antialias);
 	pango_font_description_free (bolddesc);
-	pango_font_description_free (italicdesc);
-	pango_font_description_free (bolditalicdesc);
 
 	/* Decide if we should keep this bold font face, per bug 54926:
 	 *  - reject bold font if it is not within 10% of normal font width
 	 */
-	normal = VTE_DRAW_NORMAL;
-	bold   = normal | VTE_DRAW_BOLD;
-	ratio = draw->fonts[bold]->width * 100 / draw->fonts[normal]->width;
-	if (abs(ratio - 100) > 10) {
-		_vte_debug_print (VTE_DEBUG_DRAW,
-			"Rejecting bold font (%i%%).\n", ratio);
-		font_info_destroy (draw->fonts[bold]);
-		draw->fonts[bold] = draw->fonts[normal];
-	}
-	normal = VTE_DRAW_ITALIC;
-	bold   = normal | VTE_DRAW_BOLD;
-	ratio = draw->fonts[bold]->width * 100 / draw->fonts[normal]->width;
-	if (abs(ratio - 100) > 10) {
-		_vte_debug_print (VTE_DEBUG_DRAW,
-			"Rejecting italic bold font (%i%%).\n", ratio);
-		font_info_destroy (draw->fonts[bold]);
-		draw->fonts[bold] = draw->fonts[normal];
+	if ( abs((draw->font_bold->width * 100 / draw->font->width) - 100) > 10 ) {
+		font_info_destroy (draw->font_bold);
+		draw->font_bold = draw->font;
 	}
 }
 
@@ -894,42 +945,41 @@ void
 _vte_draw_get_text_metrics(struct _vte_draw *draw,
 			   gint *width, gint *height, gint *ascent)
 {
-	g_return_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL);
+	g_return_if_fail (draw->font != NULL);
 
 	if (width)
-		*width  = draw->fonts[VTE_DRAW_NORMAL]->width;
+		*width  = draw->font->width;
 	if (height)
-		*height = draw->fonts[VTE_DRAW_NORMAL]->height;
+		*height = draw->font->height;
 	if (ascent)
-		*ascent = draw->fonts[VTE_DRAW_NORMAL]->ascent;
+		*ascent = draw->font->ascent;
 }
 
 
 int
 _vte_draw_get_char_width (struct _vte_draw *draw, vteunistr c, int columns,
-						  guint style)
+			  gboolean bold)
 {
 	struct unistr_info *uinfo;
 
-	g_return_val_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL, 0);
+	g_return_val_if_fail (draw->font != NULL, 0);
 
-	uinfo = font_info_get_unistr_info (draw->fonts[style], c);
+	uinfo = font_info_get_unistr_info (bold ? draw->font_bold : draw->font, c);
 	return uinfo->width;
 }
 
-gboolean
-_vte_draw_has_bold (struct _vte_draw *draw, guint style)
+static gboolean
+_vte_draw_has_bold (struct _vte_draw *draw)
 {
-	return (draw->fonts[style ^ VTE_DRAW_BOLD] != draw->fonts[style]);
+	return (draw->font != draw->font_bold);
 }
 
-void
-_vte_draw_set_source_color_alpha (struct _vte_draw *draw,
-                                  const PangoColor *color,
-                                  guchar            alpha)
+static void
+set_source_color_alpha (cairo_t        *cr,
+			const PangoColor *color,
+			guchar alpha)
 {
-        g_assert(draw->cr);
-	cairo_set_source_rgba (draw->cr,
+	cairo_set_source_rgba (cr,
 			      color->red / 65535.,
 			      color->green / 65535.,
 			      color->blue / 65535.,
@@ -939,18 +989,17 @@ _vte_draw_set_source_color_alpha (struct _vte_draw *draw,
 static void
 _vte_draw_text_internal (struct _vte_draw *draw,
 			 struct _vte_draw_text_request *requests, gsize n_requests,
-			 const PangoColor *color, guchar alpha, guint style)
+			 const PangoColor *color, guchar alpha, gboolean bold)
 {
 	gsize i;
 	cairo_scaled_font_t *last_scaled_font = NULL;
 	int n_cr_glyphs = 0;
 	cairo_glyph_t cr_glyphs[MAX_RUN_LENGTH];
-	struct font_info *font = draw->fonts[style];
+	struct font_info *font = bold ? draw->font_bold : draw->font;
 
 	g_return_if_fail (font != NULL);
 
-        g_assert(draw->cr);
-	_vte_draw_set_source_color_alpha (draw, color, alpha);
+	set_source_color_alpha (draw->cr, color, alpha);
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 
 	for (i = 0; i < n_requests; i++) {
@@ -1006,9 +1055,9 @@ _vte_draw_text_internal (struct _vte_draw *draw,
 void
 _vte_draw_text (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *requests, gsize n_requests,
-	       const PangoColor *color, guchar alpha, guint style)
+	       const PangoColor *color, guchar alpha, gboolean bold)
 {
-        g_assert(draw->cr);
+	g_return_if_fail (draw->started);
 
 	if (_vte_debug_on (VTE_DEBUG_DRAW)) {
 		GString *string = g_string_new ("");
@@ -1018,17 +1067,16 @@ _vte_draw_text (struct _vte_draw *draw,
 			g_string_append_unichar (string, requests[n].c);
 		}
 		str = g_string_free (string, FALSE);
-		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d), %s - %s)\n",
-				str, n_requests, color->red, color->green, color->blue, alpha,
-				(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
-				(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
+		g_printerr ("draw_text (\"%s\", len=%"G_GSIZE_FORMAT", color=(%d,%d,%d,%d), %s)\n",
+				str, n_requests, color->red, color->green, color->blue,
+				alpha, bold ? "bold" : "normal");
 		g_free (str);
 	}
 
-	_vte_draw_text_internal (draw, requests, n_requests, color, alpha, style);
+	_vte_draw_text_internal (draw, requests, n_requests, color, alpha, bold);
 
 	/* handle fonts that lack a bold face by double-striking */
-	if ((style & VTE_DRAW_BOLD) && !_vte_draw_has_bold (draw, style)) {
+	if (bold && !_vte_draw_has_bold (draw)) {
 		gsize i;
 
 		/* Take a step to the right. */
@@ -1036,7 +1084,7 @@ _vte_draw_text (struct _vte_draw *draw,
 			requests[i].x++;
 		}
 		_vte_draw_text_internal (draw, requests,
-					   n_requests, color, alpha, style);
+					   n_requests, color, alpha, FALSE);
 		/* Now take a step back. */
 		for (i = 0; i < n_requests; i++) {
 			requests[i].x--;
@@ -1045,38 +1093,35 @@ _vte_draw_text (struct _vte_draw *draw,
 }
 
 gboolean
-_vte_draw_has_char (struct _vte_draw *draw, vteunistr c, guint style)
+_vte_draw_has_char (struct _vte_draw *draw, vteunistr c, gboolean bold)
 {
 	struct unistr_info *uinfo;
 
-	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X', %s - %s)\n", c,
-				(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
-				(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
+	_vte_debug_print (VTE_DEBUG_DRAW, "draw_has_char ('0x%04X', %s)\n", c,
+			  bold ? "bold" : "normal");
 
-	g_return_val_if_fail (draw->fonts[VTE_DRAW_NORMAL] != NULL, FALSE);
+	g_return_val_if_fail (draw->font != NULL, FALSE);
 
-	uinfo = font_info_get_unistr_info (draw->fonts[style], c);
+	uinfo = font_info_get_unistr_info (bold ? draw->font_bold : draw->font, c);
 	return !uinfo->has_unknown_chars;
 }
 
 gboolean
 _vte_draw_char (struct _vte_draw *draw,
 	       struct _vte_draw_text_request *request,
-	       const PangoColor *color, guchar alpha, guint style)
+	       const PangoColor *color, guchar alpha, gboolean bold)
 {
 	gboolean has_char;
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
-			"draw_char ('%c', color=(%d,%d,%d,%d), %s, %s)\n",
+			"draw_char ('%c', color=(%d,%d,%d,%d), %s)\n",
 			request->c,
 			color->red, color->green, color->blue,
-			alpha,
-			(style & VTE_DRAW_BOLD)   ? "bold"   : "normal",
-			(style & VTE_DRAW_ITALIC) ? "italic" : "regular");
+			alpha, bold ? "bold" : "normal");
 
-	has_char =_vte_draw_has_char (draw, request->c, style);
+	has_char =_vte_draw_has_char (draw, request->c, bold);
 	if (has_char)
-		_vte_draw_text (draw, request, 1, color, alpha, style);
+		_vte_draw_text (draw, request, 1, color, alpha, bold);
 
 	return has_char;
 }
@@ -1086,7 +1131,7 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
 			 const PangoColor *color, guchar alpha)
 {
-        g_assert(draw->cr);
+	g_return_if_fail (draw->started);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
@@ -1096,7 +1141,7 @@ _vte_draw_draw_rectangle (struct _vte_draw *draw,
 
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 	cairo_rectangle (draw->cr, x+VTE_LINE_WIDTH/2., y+VTE_LINE_WIDTH/2., width-VTE_LINE_WIDTH, height-VTE_LINE_WIDTH);
-	_vte_draw_set_source_color_alpha (draw, color, alpha);
+	set_source_color_alpha (draw->cr, color, alpha);
 	cairo_set_line_width (draw->cr, VTE_LINE_WIDTH);
 	cairo_stroke (draw->cr);
 }
@@ -1106,7 +1151,7 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 			 gint x, gint y, gint width, gint height,
 			 const PangoColor *color, guchar alpha)
 {
-        g_assert(draw->cr);
+	g_return_if_fail (draw->started);
 
 	_vte_debug_print (VTE_DEBUG_DRAW,
 			"draw_fill_rectangle (%d, %d, %d, %d, color=(%d,%d,%d,%d))\n",
@@ -1116,6 +1161,6 @@ _vte_draw_fill_rectangle (struct _vte_draw *draw,
 
 	cairo_set_operator (draw->cr, CAIRO_OPERATOR_OVER);
 	cairo_rectangle (draw->cr, x, y, width, height);
-	_vte_draw_set_source_color_alpha (draw, color, alpha);
+	set_source_color_alpha (draw->cr, color, alpha);
 	cairo_fill (draw->cr);
 }
